﻿using System;
using GameKnight.Core;

namespace GameKnight
{

	namespace GameBoy
	{
	
		public partial class Processor : IProcessor, ISerializable
		{
		
			private readonly IConsole Console = null;

			private double Cycles = 0.0;

			private readonly Register.IE IE = null;

	        private readonly Register.IF IF = null;
            
			private readonly Register.HDMA HDMA = null;

			private readonly Register.KEY1 KEY1 = null;

			private readonly IMemory Memory = null;
			
			public Processor( IConsole Console )
			{
				this.Console = Console;
				this.Memory = Console.Memory;
				this.IE = Console.GetRegister<Register.IE>();
				this.IF = Console.GetRegister<Register.IF>();
				this.HDMA = Console.GetRegister<Register.HDMA>();
				this.KEY1 = Console.GetRegister<Register.KEY1>();
				if( KEY1 != null )
				{
					this.AF = 0x11B0;
					this.BC = 0x0000;
					this.DE = 0xFF56;
					this.HL = 0x000D;			
				}
				return;
			}
			
			public void Reset()
			{
				IFF = 0;
				PC = 0x100;
				BC = 0x0013;
				DE = 0x00D8;
				HL = 0x014D; 
				SP = 0xFFFE;
				AF = 0x01B0;
				Cycles = 0.0;
				return;
			}
			
			public void Run( double TimePassed )
			{
				Cycles += ( ( 4194304.0 * ( ( KEY1 != null ) ? KEY1.Speed : 1 ) ) * TimePassed );
				while( Cycles > 0.0 )
				{		
					Opcode Opcode = null;
					byte Value = Memory.ReadU8( PC++ );
					if( ( IFF & 0x100 ) != 0 )
					{
						IFF &= 0xFF;
						PC--;
					}
					if( Value != 0xCB )
					{
						Opcode = Opcode.Table[Value];
					}
					else
					{
						Value = Memory.ReadU8( PC++ );
						Opcode = Opcode.TableCB[Value];
					}
					Cycles -= (double)( Opcode.Cycles + Opcode.Function( this, Opcode ) );
					// TODO: This is stupid
					if( ( HDMA != null ) && ( HDMA.HaltFlag ) )
					{
						HDMA.HaltFlag = false;
						Cycles -= ( 231 + ( ( HDMA[5] & 0x7F ) << 4 ) );
					}
					IF.Value = (byte)Console.InterruptMask;
					if( ( IFF & 0x20 ) != 0 )
					{
						IFF &= 0xDF;
						IFF |= 0x01;
					}
					else if( ( ( IFF & 0x01 ) != 0 ) && ( Console.InterruptMask != 0 ) && ( IE.Value != 0 ) )
					{
						for( int j = 0; j < 5; j++ )
						{
							if( ( ( Console.InterruptMask & ( 1 << j ) ) != 0 ) && ( ( IE.Value & ( 1 << j ) ) != 0 ) )
							{
								if( ( IFF & 0x80 ) != 0)
								{
									PC++;
									IFF &= 0x7F;
								}
								IFF &= 0x7E;
								Console.InterruptMask &= (byte)( ~( 1 << j ) );
								IF.Value &= (byte)( ~( 1 << j ) );
								SP -= 2;
								Memory.WriteU16( SP, PC );
								PC = (ushort)( 0x0040 + ( j << 3 ) );
								break;
							}
						}
					}	
				}
				return;
			}
			
		}

	}
	
}
