using System;
using Collections = System.Collections.Generic;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
		
		public partial class VideoMemory : IVideoMemory
		{

			private class Sprite : ISprite, Collections.IComparer<Sprite>
			{

				public int Address
				{
					get
					{
						return AddressProperty;
					}
					private set
					{
	                	AddressProperty = value;
					}
				}
	
				private int AddressProperty = 0;

				private byte[] DataArray = new byte[4];

				public bool FlippedHorizontally
				{
					get
					{
						return ( ( DataArray[3] & 0x20 ) != 0 );
					}
				}
				
				public bool FlippedVertically
				{
					get
					{
						return ( ( DataArray[3] & 0x40 ) != 0 );
					}
				}
				
                public int Height
				{
					get
					{
						return LCDC.SpriteHeight;
					}
				}
				
				private readonly Register.LCDC LCDC = null;

				private readonly IPalette[] PaletteArray = new IPalette[2];

				public int PaletteBank
				{
					get
					{
						return ( ( DataArray[3] >> 4 ) & 0x01 );
					}
				}

				public int PaletteIndex
				{
					get
					{
						return ( ( PaletteArray[0].Entries > 4 ) ? ( DataArray[3] & 0x07 ) : 0 );
					}
				}

                public int PatternNumber
				{
					get
					{
						return ( ( Height <= 8 ) ? DataArray[2] : ( DataArray[2] & 0xFE ) );
					}
				}

				//private readonly IPatternTable PatternTable = null;

                public int Priority
				{
					get
					{
						return ( DataArray[3] >> 7 );
					}
				}
				
				public int TileBank
				{
					get
					{
						return ( ( DataArray[3] >> 3 ) & 0x01 );
					}
				}

                public int X
				{
					get
					{
						return ( DataArray[1] - 8 );
					}
				}

				public int Y
				{
					get
					{
						return ( DataArray[0] - 16 );
					}
				}

				public int Width
				{
					get
					{
						return 8;
					}
				}

				private readonly IVideoMemory VideoMemory = null;

				public Sprite( IConsole Console, IVideoMemory VideoMemory, int Address ) : base()
				{
					this.VideoMemory = VideoMemory;
					this.Address = Address;
					this.PaletteArray[0] = VideoMemory.GetPalette( 1 );
					this.PaletteArray[1] = ( VideoMemory.GetPalette( 2 ) ?? PaletteArray[0] );
					this.LCDC = Console.GetRegister<Register.LCDC>();
					return;
				}

				public int Compare( Sprite FirstSprite, Sprite SecondSprite )
				{
					if( FirstSprite.X == SecondSprite.X )
					{
						return ( ( FirstSprite.Address < SecondSprite.Address ) ? 1 : -1 );
					}
					return ( ( FirstSprite.X < SecondSprite.X ) ? 1 : -1 );
				}

				public void Draw( int ScanLine, int[] IndexArray, uint[] ColorArray )
				{
					int PX = 0;
					int Index = 0;
					int Offset = ( ScanLine - Y );
					int DrawPriority = Priority;
					int PaletteOffset = ( PaletteIndex << 2 );
					ushort PixelValues = VideoMemory.ReadU16( ( 0x8000 + ( TileBank << 13 ) + ( PatternNumber << 4 ) ) + ( FlippedVertically ? ( ( ( Height - 1 ) - Offset ) << 1 ) : ( Offset << 1 ) ) );
					IPalette Palette = PaletteArray[PaletteBank];
					for( int i = 0; i < 8; i++ )
					{
						Index = ( ( ( PixelValues >> i ) & 0x01 ) | ( ( PixelValues >> ( i + 7 ) ) & 0x02 ) );
						if( Index != 0 )
						{
							Index |= PaletteOffset;
							PX = ( FlippedHorizontally ? ( X + i ) : ( X + ( 7 - i ) ) );
							if( ( PX >= 0 ) && ( PX < ColorArray.Length ) )
							{
								PX |= ( ScanLine << 8 );
								if( DrawPriority != 0 )
								{
									if( IndexArray[PX] == 0 )
									{
										ColorArray[PX] = Palette.ColorArray[Index];
									}
								}
								else
								{
									ColorArray[PX] = Palette.ColorArray[Index];
								}
							}
						}
					}
					return;
				}

				public byte this[int Index]
				{
					get
					{
						return DataArray[Index&0x03];
					}
					set
					{
						DataArray[Index&0x03] = value;	
					}
				}
	
			}
				
		}

	}
	
}
