using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
		
		public partial class VideoMemory : IVideoMemory
		{
		
			private class Tile : ITile
			{
	
				public int Address
				{
					get
					{
						return AddressProperty;
					}
					private set
					{
	                	AddressProperty = value;
					}
				}
	
				private int AddressProperty = 0;
	
				private static readonly byte[][] DataArray = new byte[0x10000][];
				
				public int Height
				{
					get
					{
						return 8;
					}
				}
				
				private readonly IPatternTable PatternTable = null;
				
				private readonly IPalette Palette = null;
				
				private readonly IVideoMemory VideoMemory = null;
	
				public int Width
				{
					get
					{
						return 8;
					}
				}
				
				static Tile()
				{
					for( int i = 0; i < DataArray.Length; i++ )
					{
						DataArray[i] = new byte[8];
						for( int j = 0; j < 8; j++ )
						{
							DataArray[i][7-j] = (byte)( ( ( i >> j ) & 0x01 ) | ( ( i >> ( j + 7 ) ) & 0x02 ) );
						}
					}
					return;
				}
				
				public Tile( IVideoMemory VideoMemory, IPatternTable PatternTable, int Address )
				{
					this.VideoMemory = VideoMemory;
					this.PatternTable = PatternTable;
					this.Palette = VideoMemory.GetPalette( 0 );
					this.Address = Address;
					return;
				}
	
				public void Draw( int X, int ScanLine, int Offset, ITileAttributes TileAttributes, int[] IndexArray, uint[] ColorArray )
				{
					int i = 0;
					int TileAddress = ( Address + ( TileAttributes.FlippedVertically ? ( ( 7 - Offset ) << 1 ) : ( Offset << 1 ) ) );
					int PixelValues = ( PatternTable[TileAddress] | ( PatternTable[TileAddress+1] << 8 ) );
					int PaletteIndex = ( TileAttributes.PaletteIndex << 2 );
					if( X < 0 )
					{
						i = -X;
						X = 0;
					}
					X |= ( ScanLine << 8 );
					if( TileAttributes.FlippedHorizontally )
					{
						for( ; i < 8; i++, X++ )
						{
							IndexArray[X] = DataArray[PixelValues][7-i];
							ColorArray[X] = Palette.ColorArray[IndexArray[X]|PaletteIndex];
						}
					}
					else
					{
						for( ; i < 8; i++, X++ )
						{
							IndexArray[X] = DataArray[PixelValues][i];
							ColorArray[X] = Palette.ColorArray[IndexArray[X]|PaletteIndex];
						}
					}
					return;
				}
	
			}
				
		}
		
	}
	
}
