﻿using System;
using System.Runtime.InteropServices;
using Collections = System.Collections.Generic;
using GameKnight.Core;

namespace GameKnight.Win32
{

	public sealed class Timer : ITimer
	{

		[DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceCounter( out long Count );

		[DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceFrequency( out long Frequency );

		private static long Count = 0;
		
		private static long Frequency = 0;

		private static long LastCount = 0;

		private readonly Collections.Dictionary<object,double> ObjectList = new Collections.Dictionary<object,double>();
		
		private static bool Paused = false;

		public double TimePassed
		{
			get
			{
				return TimePassedProperty;
			}
			private set
			{
				TimePassedProperty = value;
			}
		}
	
		private double TimePassedProperty = 0.0;

		static Timer()
		{
			if( QueryPerformanceFrequency( out Frequency ) == false )
			{
				throw new Exception( "QueryPerformance functions not supported." );
			}
			return;
		}
		
		public Timer()
		{
			Reset();
			return;
		}
				
		public double GetInterval( object Item )
		{
			if( ObjectList.ContainsKey( Item ) == false )
			{
				ObjectList.Add( Item, TimePassed );
			}
			if( Paused == false )
			{
				LastCount = Count;		
				QueryPerformanceCounter( out Count );
				TimePassed += ( (double)( Count - LastCount ) / (double)Frequency );
			}		
			double Result = ( TimePassed - ObjectList[Item] );
			ObjectList[Item] = TimePassed;
			return Result;
		}

		public void Pause()
		{                       
			Paused = true;
			return;
		}

		public void Reset()
		{              
			ObjectList.Clear();
			TimePassed = 0.0;
			QueryPerformanceCounter( out Count );
			LastCount = Count;
			return;
		}
			
		public void Resume()
		{
			Paused = false;
			QueryPerformanceCounter( out Count );
			LastCount = Count;
			return;
		}

	}

}
